/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common;
/*
 * Revision 1.5  2007/09/07 09:49:55  Trier
 * <Kein Kommentar eingegeben>
 */
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * <p>Stellt eine Methode zur Ausgabe von ResultData zur Verf&uuml;gung, die
 * deren Ausgabe kompakter ist als ResultData.toString().</p>
 *
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: DebugHelper.java 1292 2009-03-20 13:12:46Z Trier $
 */
abstract public class DebugHelper {

  private static DateFormat DATEFORMAT_DDMMJJJJ_HHMMSS = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  /**
   * <p>Hilfsmethode zum Erzeugen eines Strings mit kommagetrennten Pids der in
   * der &uuml;bergebenen Collection enthaltenen SystemObjekte.</p>
   *
   * @param collection Collection
   * @return String mit kommagetrennten Pids oder EmptyCollection oder null.
   */
  public static <T extends SystemObject> String toString(Collection<T> collection) {
    if (collection==null) {
      return "null";
    }
    if (collection.size()==0) {
      return "EmptyCollection";
    }
    StringBuilder sb = new StringBuilder();
    boolean first = true;
    for (T so: collection) {
      if (first) {
        sb.append(so.getPid());
        first = false;
      }
      else {
        sb.append(", ").append(so.getPid());
      }
    }
    return sb.toString();
  }


  public static String getHeader(ResultData rs) {
    if (rs==null) {
      return null;
    }
    SystemObject so = rs.getObject();
    DataDescription descr = rs.getDataDescription();
    Date date = new Date(rs.getDataTime());
    StringBuilder sb = new StringBuilder();
    sb.append(DATEFORMAT_DDMMJJJJ_HHMMSS.format(date)).append(" ");
    sb.append(so.getPid()).append(':');
    sb.append(descr.getAttributeGroup().getPid()).append(':');
    sb.append(descr.getAspect().getPid()).append(':');
    sb.append(descr.getSimulationVariant()).append(' ');
    sb.append("delayed=").append(rs.isDelayedData());
    return sb.toString();
  }

  public static String toString(Dataset rs) {
    if (rs==null) {
      return null;
    }
    SystemObject so = rs.getObject();
    DataDescription descr = rs.getDataDescription();
    Date date = new Date(rs.getDataTime());
    StringBuilder sb = new StringBuilder();
    sb.append(DATEFORMAT_DDMMJJJJ_HHMMSS.format(date)).append(" ");
    sb.append(so.getPid()).append(':');
    sb.append(descr.getAttributeGroup().getPid()).append(':');
    sb.append(descr.getAspect().getPid()).append(':');
    sb.append(descr.getSimulationVariant()).append(' ');
    sb.append("dataType=").append(rs.getDataType()).append(' ');
    sb.append("dataKind=").append(rs.getDataKind()).append(' ');
    if (rs instanceof ArchiveData) {
      sb.append("ArchiveTime=").append(((ArchiveData)rs).getArchiveTime()).append(' ');
    }
    while (sb.length()<114) {
      sb.append(' ');
    }
    if (rs.getData()!=null) {
      appendData(sb, rs.getData());
    }
    else {
      sb.append( "noData" );
    }
    return sb.toString();
  }

  public static String toString(final SystemObject[] soArray) {
    if (soArray==null) {
      return "nullArray";
    }
    if (soArray.length==0) {
      return "Array[]";
    }
    StringBuilder sb = new StringBuilder();
    sb.append('[').append(soArray[0].getPid());
    for (int i=1; i< soArray.length; i++) {
      sb.append(", ").append(soArray[i].getPid());
    }
    sb.append(']');
    return sb.toString();
  }

  public static String print(Data data) {
    StringBuilder sb = new StringBuilder();
    appendData(sb, data);
    return sb.toString();
  }

  private static void appendData(StringBuilder sb, Data data) {
    if (data.isList()) {
      AttributeType attr = data.getAttributeType();

      if ((attr!=null) && ("atl.status".equals(attr.getPid()))) {
        appendJNValue(sb, data.getItem("Erfassung").getItem("NichtErfasst"), "nErf");
        appendJNValue(sb, data.getItem("PlFormal").getItem("WertMax"), "wMax");
        appendJNValue(sb, data.getItem("PlFormal").getItem("WertMin"), "wMin");
        appendJNValue(sb, data.getItem("PlLogisch").getItem("WertMaxLogisch"), "wMaL");
        appendJNValue(sb, data.getItem("PlLogisch").getItem("WertMinLogisch"),"wMiL");
        appendJNValue(sb, data.getItem("MessWertErsetzung").getItem("Implausibel"), "Impl");
        appendJNValue(sb, data.getItem("MessWertErsetzung").getItem("Interpoliert"), "Intp");
      }
      else {
        if ((attr!=null) && ("atl.gte".equals(attr.getPid()))) {
          sb.append(data.getItem("Index").asTextValue().getValueText());
        }
        else {
          Iterator<?> it = data.iterator();
          if (it==null) {
            sb.append(" ").append(data.getName()).append("==null ");
          }
          else {
            while (it.hasNext()) {
              Data subData = (Data)it.next();
              sb.append(" ").append(subData.getName()).append("[");
              appendData(sb, subData);
              sb.append("] ");
            }
          }
        }
      }
    }
    else {
      if (data.isPlain()) {
        String s = data.asTextValue().getValueText();
        for (int i=0; i<(5-s.length()); i++) {
          sb.append(' ');
        }
        sb.append(data.asTextValue().getValueText());
      }
      else {
        if (data.isArray()) {
          Data.Array dArray = data.asArray();
          int iLength = dArray.getLength();
          if (iLength>0) {
            appendData(sb, dArray.getItem(0));
          }
          for (int i=1; i<iLength; i++) {
            sb.append(", ");
            appendData(sb, dArray.getItem(i));
          }
        }
      }
    }
  }

//  private static Boolean getJNValue(Data data) {
//     Data.NumberValue nv = data.asUnscaledValue();
//     if (nv==null) {
//       return null;
//     }
//     else {
//       try {
//         return (nv.intValue() == 1);
//       }
//       catch (Exception e) {
//         return null;
//       }
//     }
//  }

//  class Status {
//    Boolean _nErf;
//    Boolean _wMax;
//    Boolean _wMin;
//    Boolean _wMaL;
//    Boolean _wMiL;
//    Boolean _Impl;
//    Boolean _Intp;
//
//    Status(Data data) {
//      _nErf = getJNValue(data.getItem("Erfassung").getItem("NichtErfasst"));
//      _wMax = getJNValue(data.getItem("PlFormal").getItem("WertMax"));
//      _wMin = getJNValue(data.getItem("PlFormal").getItem("WertMin"));
//      _wMaL = getJNValue(data.getItem("PlLogisch").getItem("WertMaxLogisch"));
//      _wMiL = getJNValue(data.getItem("PlLogisch").getItem("WertMinLogisch"));
//      _Impl = getJNValue(data.getItem("MessWertErsetzung").getItem("Implausibel"));
//      _Intp = getJNValue(data.getItem("MessWertErsetzung").getItem("Interpoliert"));
//    }
//
//  }

  private static void appendJNValue(StringBuilder sb, Data data, String s) {
    Data.NumberValue nv = data.asUnscaledValue();
    if (nv==null) {
      sb.append(data.getName()).append("==null ");
    }
    else {
      // System.out.println(data +" nv=" + nv);
      try {
        if (nv.intValue() == 1) {
          sb.append(s).append(" ");;
        }
      }
      catch (Exception e) {
        sb.append(s).append("=null ");
      }
    }
  }


  public static String getDataIndexAsString(long dataIndex) {
    return (dataIndex >> 32) + "#" + ((dataIndex >> 2) & 0x3fffffff) + "#" + (dataIndex & 0x3);
  }


}
