/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.kex.bast.common.datawrapper;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.sys.funclib.debug.Debug;

/*
 * Revision 1.3  2008/09/04 12:52:13  Logosdev
 * Prfungen mit isState() und isNumber() von getUnscaledValue() auf getScaledValue() umgestellt, da 'Unscaled Values' fr isNumber() immer true liefern (weil auch Zustnde als Zahl ausgelesen werden knnen, was zu Fehlern beim BASt-Export in der DAUZ ST fhrte)
 *
 */

/**
 * kapselt die AttributListe atl.gte.
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author J. Schawaller
 * @version $Id: AtlGuete.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class AtlGuete {

  private Data _Data;

  /** DebugLogger fuer Debug-Ausgaben. */
  private static Debug debug = Debug.getLogger();

  public static final String ATTR_NAME_INDEX = "Index";
  public static final String ATTR_NAME_VERFAHREN = "Verfahren";

  public static final String ZUSTAND_INDEX_NICHT_ERMITTELBAR = "nicht ermittelbar";
  public static final String ZUSTAND_INDEX_FEHLERHAFT = "fehlerhaft";
  public static final String ZUSTAND_INDEX_NICHT_ERMITTELBAR_FEHLERHAFT = "nicht ermittelbar/fehlerhaft";
  public static final String ZUSTAND_VERFAHREN_STANDARD = "Standard";

  public AtlGuete(Data data) {
    _Data = data;
  }

  protected void doInitialize() {
    setIndex((byte)1);
    _Data.getScaledValue("Verfahren").setText(ZUSTAND_VERFAHREN_STANDARD);
  }

  public IntegerValueRange getRangeOfGueteIndex() {
    return ValueRangeHashtable.getInstance(this._Data).get("att.gteIndex");
  }
  public IntegerValueRange getRangeOfGueteVerfahren() {
    return ValueRangeHashtable.getInstance(this._Data).get("att.gteVerfahren");
  }

	/**
	 * Abfangen des Fehlerfalles, dass der Gteindex nicht definiert ist. Dann ist der Wert "nicht ermittelbar/fehlerhaft"
	 * fr diesen Fall wird der Gteindex = 0.0 gesetzt.
 	 * @return  Gteindex, wenn dieser definiert ist, 0.0 sonst.
	 * nderung durch inovat.
	 */
  public double getIndex() {
		if(_Data.getScaledValue("Index").isNumber()) {
			return _Data.getScaledValue("Index").doubleValue();
		} else {
			 debug.finer(String.format("Der Gteindex ist nicht ermittelbar/fehlerhaft, der Wert wird auf 0.0 gesetzt.\"%s\")", _Data.getScaledValue("Index")));
			return 0.0;
		}
  }

  public void setIndex (byte index) {
      _Data.getScaledValue("Index").set(index);
  }
  public IntegerValueState getZustandIndex() {
    return _Data.getScaledValue("Index").getState();
  }

  // Also: _Data.getUnscaledValue("Index").isNumber() liefert IMMER true, da sich die Zustnde ja als Zahl abfragen lassen
  //       _Data.getUnscaledValue("Index").isState()  liefert true falls Zustand
  // eigentlich geht es aber um _Data.getScaledValue("Index") also:
  //       _Data.getScaledValue("Index").isNumber() liefert true, wenn es wirklich ein Wert ist
  //       _Data.getScaledValue("Index").isState()  liefert true, wenn es ein Zustand ist

  public boolean isState() {
//    return _Data.getUnscaledValue("Index").isState();
    return _Data.getScaledValue("Index").isState();
  }
  public boolean isNumber() {
//    return !isState();
    return _Data.getScaledValue("Index").isNumber();
  }


  public void setZustandIndex (IntegerValueState integerValueState) {
      _Data.getScaledValue("Index").setState(integerValueState);
  }
  public void setZustandIndex (String zustandsName) {
      _Data.getScaledValue("Index").setText(zustandsName);
  }

  public byte getVerfahren() {
    return _Data.getScaledValue("Verfahren").byteValue();
  }

  public void setVerfahren (byte verfahren) {
      _Data.getScaledValue("Verfahren").set(verfahren);
  }
  public IntegerValueState getZustandVerfahren() {
    return _Data.getScaledValue("Verfahren").getState();
  }

  public void setZustandVerfahren (IntegerValueState integerValueState) {
      _Data.getScaledValue("Verfahren").setState(integerValueState);
  }
  public void setZustandVerfahren (String zustandsName) {
      _Data.getScaledValue("Verfahren").setText(zustandsName);
  }

}

