/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;

/*
 * Revision 1.3  2008/09/01 08:17:32  Schawaller
 * fsArray wird geklont bei get-Mothode
 */
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.*;
import de.inovat.kex.bast.common.*;

import java.util.*;



/**
 * kapselt den DatenModellTyp messQuerschnitt
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: MessQuerschnitt.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class MessQuerschnitt
    extends ConfigurationObjectWrapper {

  private static WeakHashMap<String,MessQuerschnitt> __Cache = new WeakHashMap<String,MessQuerschnitt>();

  private static ConfigurationObjectType __ConfigurationObjectType;
  private static AttributeGroup __AtgMessQuerschnittAllgemein;

  private AuswerteQuerschnitt _AuswerteQuerschnitt;
  private String _ErsatzMessQuerschnittPID;
  private String _Typ;

  private Fahrstreifen[] _Fahrstreifen;


  /**
   * Gibt die zugehrigen Fahrstreifen sortiert von aussen nach innen (HFS, &Uuml;FS01, &Uuml.FS02 ..) zur&uuml;ck.
   *
   * @return Fahrstreifen[]
   */
  public Fahrstreifen[] getFahrstreifen() {
    return _Fahrstreifen.clone();
  }


  /**
   * gibt den ErsatzMessQuerschnitt zurck, wenn dieser definiert wurde.
   *
   * @throws DataModellObjectNotFoundException
   * @throws DataModellMismatchException
   * @return MessQuerschnitt
   */
  public MessQuerschnitt getErsatzMessQuerschnitt()
      throws DataModellObjectNotFoundException, DataModellMismatchException {
    if (_ErsatzMessQuerschnittPID != null) {
      return getInstance(_ErsatzMessQuerschnittPID);
    }
    else {
      return null;
    }
  }


  private MessQuerschnitt(String pid) throws DataModellMismatchException, DataModellObjectNotFoundException {
    super(pid);
  }

  private MessQuerschnitt(ConfigurationObject confObj) throws DataModellMismatchException {
    super(confObj);
  }

  public void setAuswerteQuerschnitt(AuswerteQuerschnitt aq) {
    _AuswerteQuerschnitt = aq;
  }

  /**
   * gibt die AuswerteQuerschnitt zur&uuml;ck, wenn dieses Objekt durch eine
   * AuswerteQuerschnitt erzeugt wurde.
   *
   * @return AuswerteQuerschnitt
   */
  public AuswerteQuerschnitt getAuswerteQuerschnitt() {
    return _AuswerteQuerschnitt;
  }

	/**
	 * 
	 * @param confObj Konfigurationsobjekt.
	 * @throws DataModellMismatchException
	 * Erweiterung durch Dagmar Rostek (inovat)
	 * Referenz auf den ErsatzMessQuerschnitt ist optional,
	 * oData.getReferenceValue("ErsatzMessQuerschnitt").getSystemObject() = null mglich,
	 * Abfangen der NullPointerException.
	 */
  protected void initData(ConfigurationObject confObj) throws DataModellMismatchException {

    Data oData = confObj.getConfigurationData(__AtgMessQuerschnittAllgemein);
    if (oData != null) {
	   if(oData.getReferenceValue("ErsatzMessQuerschnitt").getSystemObject()!=null){
      _ErsatzMessQuerschnittPID = oData.getReferenceValue("ErsatzMessQuerschnitt").getSystemObject().getPid();
	   } 
      if (oData.getUnscaledValue("Typ").isState()) {
        _Typ = oData.getUnscaledValue("Typ").getState().getName();
      }

    }

    List<SystemObject> listFS = confObj.getObjectSet("FahrStreifen").getElements();
    _Fahrstreifen = new Fahrstreifen[listFS.size()];
    for (int fsNr = 0; (fsNr < listFS.size()); fsNr++) {
      try {
        _Fahrstreifen[fsNr] = Fahrstreifen.getInstance( (ConfigurationObject)listFS.get(fsNr));
        _Fahrstreifen[fsNr].setMessQuerschnitt(this);
      }
      catch (Exception e) {
//        System.err.println(e);
        e.printStackTrace();
      }
    }
    Arrays.sort(_Fahrstreifen, new Fahrstreifen.FSComperatorVonAussenNachInnen());
    __Cache.put(this.getPID(), this);
  }

  /**
   * gibt die Instance der MessQuerschnitt mit der PID zurck. Es wird eine
   * DataModellObjectNotFoundException geworfen, wenn die PID nicht vorhanden
   * ist oder das zur PID gehrende Objekt nich vom typ messQuerschnitt ist.
   * Eine DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
   * falsche Typen verweisen bzw. diese nicht gesetzt sind.
   *
   * @param pid String
   * @throws DataModellMismatchException
   * @return MessQuerschnitt
   */
  public static MessQuerschnitt getInstance(String pid) throws DataModellMismatchException {
    MessQuerschnitt retValue = __Cache.get(pid);
    if (retValue == null) {
      Object o = NetzClientDavConnection.getInstance().getDataModel().getObject(pid);
      if ((o != null) && (o instanceof ConfigurationObject)) {
        retValue = MessQuerschnitt.getInstanceIntern((ConfigurationObject)o);
      }
    }
    return retValue;
  }

  public static MessQuerschnitt getInstance(ConfigurationObject messQuerschnitt) throws DataModellMismatchException {
    MessQuerschnitt retValue = __Cache.get(messQuerschnitt.getPid());
    if (retValue == null) {
      retValue = MessQuerschnitt.getInstanceIntern(messQuerschnitt);
    }
    return retValue;
  }

  private static synchronized MessQuerschnitt getInstanceIntern(ConfigurationObject messQuerschnitt) throws DataModellMismatchException {
    MessQuerschnitt retValue = __Cache.get(messQuerschnitt.getPid());
    if (retValue == null) {
      if (__ConfigurationObjectType == null) {
        MessQuerschnitt.init();
      }
      retValue = new MessQuerschnitt(messQuerschnitt);
    }
    return retValue;
  }

  /**
   * gibt die DatenModellObjekte vom typ messQuerschnitt zurck, wenn keiner
   * dieser Objekte Fehler in den ObjektReferenzen aufweist.
   * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf
   * falsche Typen verweisen bzw. diese nicht gesetzt sind.
   *
   * @throws DataModellMismatchException
   * @return MessQuerschnitt[]
   */
  public static MessQuerschnitt[] getAll() throws DataModellMismatchException {
    if (__ConfigurationObjectType == null) {
      init();
    }
    MessQuerschnitt[] retValue;
    if (__ConfigurationObjectType != null) {
      List<SystemObject> list = __ConfigurationObjectType.getElements();
      retValue = new MessQuerschnitt[list.size()];
      for (int i = 0; i < list.size(); i++) {
        retValue[i] = MessQuerschnitt.getInstanceIntern((ConfigurationObject) list.get(i));
      }
    }
    else {
      retValue = new MessQuerschnitt[0];
    }
    return retValue;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("Messquerschnitt - PID: \"").append(this.getPID()).append("\"  ");
    sb.append("Name: \"").append(this.getName());
    return sb.toString();
  }

  public String getMultilineRepresentation(int indent) {
    String lf = System.getProperty("line.separator");
    StringBuffer sIndent = new StringBuffer();
    for (int i=0; i<indent; i++) {
      sIndent.append(' ');
    }
    StringBuffer sb = new StringBuffer();
    sb.append(sIndent).append("PID: \"").append(this.getPID()).append("\"").append(lf);
    sb.append(sIndent).append("Name: \"").append(this.getName()).append("\"").append(lf);
    sb.append(sIndent).append("Typ: \"").append(_Typ).append("\"").append(lf);
    if (_ErsatzMessQuerschnittPID != null) {
      sb.append(sIndent).append("ErsatzMessQuerschnittPID: \"").append(_ErsatzMessQuerschnittPID).append("\"").append(lf);
    }
    else {
      sb.append(sIndent).append("\"  _ErsatzMessQuerschnittPID ist null\"").append(lf);
    }
    sb.append(sIndent).append("  Fahrstreifen: ").append(lf);
    if ((_Fahrstreifen != null) && (_Fahrstreifen.length > 0)) {
      for (int i=0; i<_Fahrstreifen.length; i++) {
        sb.append(sIndent).append("  Fahrstreifen " + (i+1) + ": ").append(lf);
        sb.append(_Fahrstreifen[i].getMultilineRepresentation(4 + indent)).append(lf);
      }
    }
    else {
      sb.append("\"  keine Fahrstreifen vorhanden");
    }
    return sb.toString();

  }

  public String getMultilineRepresentation() {
    return getMultilineRepresentation(0);
  }

  public String getTreeString() {
    return this.getName();
  }

  ConfigurationObjectType getType() {
    return __ConfigurationObjectType;
  }

  private static synchronized void init() {
    if (__ConfigurationObjectType==null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __ConfigurationObjectType = (ConfigurationObjectType)model.getType("typ.messQuerschnitt");
      __AtgMessQuerschnittAllgemein = model.getAttributeGroup("atg.messQuerschnittAllgemein");
      List<SystemObject> list = __ConfigurationObjectType.getElements();
      model.getConfigurationData(list, __AtgMessQuerschnittAllgemein);
    }
  }

}
