/*
 * Copyright (c) 2010 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.bast.exporter.FTPDataChannel
 *
 * de.inovat.kex.bast.exporter.FTPDataChannel is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.bast.exporter.FTPDataChannel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.bast.exporter.FTPDataChannel; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Kln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.bast.exporter;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.sys.funclib.debug.Debug;
import de.inovat.kex.bast.SendeBAStStatusDaten;
import de.inovat.kex.bast.common.ChannelException;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;

import java.io.BufferedOutputStream;
import java.io.IOException;
import java.util.ArrayList;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Die Klasse stellt alle Methoden bereit um die erzeugten BASt-Band Dateien auf den FTP-Server zu schreiben
 * und existierende Dateien vom FTP-Server zu lesen.
 * <p/>
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dagmar Rostek (DR)
 * @version $Revision: 664 $ / $Date: 2012-07-16 10:19:10 +0200 (Mo, 16 Jul 2012) $ / ($Author: LG $)
 */
public class FTPDataChannel {

    /** Debug fr Debug-Ausgaben. */
    private static Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    /** Statusinformationen die an den Datenverteiler gesendet werden. */
    private SendeBAStStatusDaten _bastStatusDaten;

    /** Dateiname. */
    private String _fileName;

    /** FTPClient fr die FTP Connetion. */
    private FTPClient _ftpClient;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erstellt eine Instanz der Klasse.
     *
     * @param fileName Dateiname.
     * @param bastStatusDaten Statusinformationen.
     */
    public FTPDataChannel(String fileName, SendeBAStStatusDaten bastStatusDaten) {
        _fileName        = fileName;
        _bastStatusDaten = bastStatusDaten;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode wechselt in das Exportverzeichnis fr den gestarteten Export.
     *
     * @param unterordnerName Unterordner fr die FTP-Datei oder null.
     *
     * @return true, wenn der Wechsel in das Exportverzeichnis auf dem FTP-Server ohne
     * Exception geklappt hat, false sonst.
     *
     * @throws ChannelException ChannelException.
     */
    public boolean changeExportDirektoryFTPServer(String unterordnerName) throws ChannelException {
        boolean changeDirectoryOK = false;

        // -------------------------------------------------------------------
        // Definiere das Exportverzeichnis:
        String ordner = FTPServerConnection.getInstance().getFtpPathExport();

        if (unterordnerName != null) {
            ordner = String.format("%s/%s", ordner, unterordnerName);
        }

        //
        String infoText = String.format("Es ist nicht mglich in das korrekte Verzeichnis auf dem FTP-Server <%s> zu wechseln. Bitte kontrollieren Sie die Angaben.", ordner);

        // -------------------------------------------------------------------
        // ffne das Exportverzeichnis und Unter-Verzeichnis (falls angegeben).
        // Falls nicht klappt, erstelle das Unter-Verzeichnis und versuche noch mal.
        try {
            changeDirectoryOK = _ftpClient.changeWorkingDirectory(ordner);

            // falls nicht klappt:
            if (!changeDirectoryOK && (unterordnerName != null)) {
                if (_ftpClient.makeDirectory(ordner)) {
                    changeDirectoryOK = _ftpClient.changeWorkingDirectory(ordner);
                } else {
                    debug.error(String.format("Verzeichnis <%s> kann nicht erstellt werden.", ordner));
                }
            }
        }
        catch (IOException e) {
            debug.error(infoText, e);
        }

        if (!changeDirectoryOK) {
            throw new ChannelException(infoText);
        }

        return changeDirectoryOK;
    }

    /**
     * Die Methode verbindet den FTPClienten mit dem FTP-Server.
     * @return FTPClient, wenn die Verbindung ohne Exception geklappt hat, null sonst.
     *
     * @throws ChannelException ChannelException.
     */
    public FTPClient connectToFTPServerExport() throws ChannelException {
        _ftpClient = new FTPClient();

        String infoText = "Es konnte keine Verbindung zum FTP-Server hergestellt werden. Bitte kontrollieren Sie die URL!";

        try {
            _ftpClient.connect(FTPServerConnection.getInstance().getFtpServerNameExport());
        }
        catch (IOException e) {
            debug.error(infoText, e);

            throw new ChannelException("Es ist eine Ausnahme aufgetreten: " + e);
        }

        if (_ftpClient.isConnected()) {
            return _ftpClient;
        } else {
            throw new ChannelException(infoText);
        }
    }

    /**
     * Die Methode lscht die bergebene Datei vom FTP-Server.
     * @param fileName Datei die vom FTP-Server gelscht wird.
     * @return true wenn die bergebene Datei gelscht wurde, false sonst.
     */
    public boolean dateiLoeschen(String fileName) {
        try {
            return _ftpClient.deleteFile(fileName);
        }
        catch (IOException e) {
            debug.info("Die Datei " + fileName + " konnte nicht gelscht werden." + e);
        }

        return false;
    }

    /**
     * Die Methode erzeugt den gepufferten Ausgabestrom auf den FTP-Server.
     * @return  BufferedOutputStream fr den Exportdatensatz.
     */
    public BufferedOutputStream erzeugeAusgabeStrom() {
        try {
            BufferedOutputStream buf = new BufferedOutputStream(_ftpClient.appendFileStream(_fileName));

            pruefeFTPBefehlAusfuehrung("FEHLER beim Erstellen der Ausgabedatei.");

            return buf;
        }
        catch (IOException e) {
            debug.error("Die Datei konnte nicht auf den FTP-Server geschrieben werden.", e);
        }

        return null;
    }

    /**
     * Die Methode fhrt das Login auf dem FTP-Server durch.
     * @return true, wenn das Login geklappt hat, false sonst.
     *
     * @throws ChannelException ChannelException.
     */
    public boolean loginToFTPServerExport() throws ChannelException {
        boolean loginOK;
        String  infoText = "Ein login auf dem FTP-Server war nicht mglich. Bitte kontrollieren Sie die Angaben Benutzer und Passwort.";

        try {
            loginOK = _ftpClient.login(FTPServerConnection.getInstance().getFtpBenutzerNameExport(), FTPServerConnection.getInstance().getFtpPasswdExport());

            if (FTPServerConnection.getInstance().isFtpPassivModus()) {

                // NLStBV, HEINZE, 09.05.2012: use passive mode (every time a download/upload start)
                _ftpClient.enterLocalPassiveMode();
            }
        }
        catch (IOException e) {
            debug.error(infoText, e);

            throw new ChannelException(infoText);
        }

        if (!loginOK) {
            debug.error(infoText);

            throw new ChannelException(infoText);
        }

        return loginOK;
    }

    /**
     * Prfe FTP-Befehl Abschluss.
     *
     * @param meldung Fehlermeldung.
     */
    private void pruefeFTPBefehlAbschluss(String meldung) {
        try {
            boolean c = _ftpClient.completePendingCommand();

            if (!c) {
                debug.error(meldung);
                sendeBAStStatusDaten("", meldung);
            }
        }
        catch (IOException e) {
            debug.error("Fehler bei der Prfung des FTP-Befehl Abschlusses", e);
        }
    }

    /**
     * Prfe FTP-Befehl-Ausfhrung.
     *
     * @param meldung Fehlermeldung.
     */
    private void pruefeFTPBefehlAusfuehrung(String meldung) {
        boolean r = FTPReply.isPositivePreliminary(_ftpClient.getReplyCode());

        if (!r) {
            debug.error(meldung);
            sendeBAStStatusDaten("", meldung);
        }
    }

    /**
     * Die Methode prft ob die bergebene Datei auf dem FTP-Server existiert.
     * @param fileName   Filename der bzgl. Existenz berprft wird.
     * @return true, wenn die Datei existiert, false sonst.
     */
    public boolean pruefeObFileExistiert(String fileName) {
        boolean fileExistiert = false;

        try {
            String[] alleFiles = _ftpClient.listNames();

            for (String file : alleFiles) {
                if (file.equals(fileName)) {
                    fileExistiert = true;
                }
            }
        }
        catch (IOException e) {
            debug.error("Es sind Probleme beim einlesen der Daten aufgetreten.", e);
        }

        return fileExistiert;
    }

    /**
     * Die Methode schreibt die Datei auf den FTP-Server.
     * @param bufferedAusgabe Ausgabestrom.
     * @param lineBuffer Array, dass den Inhalt des Datenexportes enthlt.
     * @throws de.inovat.kex.bast.common.ChannelException  Exception.
     */
    public void schreibeDateiAufFTPServer(BufferedOutputStream bufferedAusgabe, ArrayList<Object> lineBuffer) throws ChannelException {
        if (FTPServerConnection.getInstance().isAchslastExport()) {
            if (lineBuffer.size() > 0) {
                while ((lineBuffer.size() > 0)) {
                    try {

                        // Notwendig um die Daten im Binrformat auf den Server zu schreiben.
                        byte[] b;

                        b = (new byte[] { (Byte) (lineBuffer.remove(0)) });
                        bufferedAusgabe.write(b);
                        pruefeFTPBefehlAusfuehrung("FEHLER beim Schreiben in die Datei.");
                    }
                    catch (IOException e) {
                        String errorText = "Die Datei lsst sich nicht auf dem FTP-Server erzeugen.";

                        debug.error(errorText, e);
                        sendeBAStStatusDaten("", String.format("%s %s", errorText, e));
                    }
                }

                try {
                    bufferedAusgabe.close();
                    pruefeFTPBefehlAbschluss("FEHLER beim Schlieen der Datei");
                }
                catch (IOException e) {
                    String errorText = "Es sind Probleme beim Schlieen der Datei auf dem FTP-Server aufgetreten.";

                    debug.error(errorText, e);
                    sendeBAStStatusDaten("", String.format("%s %s", errorText, e));
                }
            }

            sendeBAStStatusDaten(FTPServerConnection.getInstance().bestimmeProzentExport(0), "");
        } else {
            int allLinesEinesExportes = lineBuffer.size();

            try {
                while ((lineBuffer.size() > 0)) {

                    //
                    bufferedAusgabe.write(lineBuffer.remove(0).toString().getBytes());
                    pruefeFTPBefehlAusfuehrung("FEHLER beim Schreiben in die Datei.");

                    //
                    bufferedAusgabe.write(System.getProperty("line.separator").getBytes());
                    pruefeFTPBefehlAusfuehrung("FEHLER beim Schreiben in die Datei.");

                    //
                    sendeBAStStatusDaten(FTPServerConnection.getInstance().bestimmeProzentExport(allLinesEinesExportes), "");
                }

                bufferedAusgabe.close();
                pruefeFTPBefehlAbschluss("FEHLER beim Schlieen der Datei");
            }
            catch (IOException e) {
                String errorText = "Die Datei konnte nicht auf den FTP-Server geschrieben werden.";

                debug.error(errorText, e);
                sendeBAStStatusDaten("", String.format("%s %s", errorText, e));
            }
        }
    }

    /**
     * Senden des aktuellen Status an GUI.
     *
     * @param prozent       Prozentangabe des aktuellen Exportvorganges oder leeres String.
     * @param statusmeldung Statusmeldung zum aktuellen Exportvorgang.
     */
    private void sendeBAStStatusDaten(String prozent, String statusmeldung) {
        try {
            _bastStatusDaten.sendeBAStStatusDaten(prozent, statusmeldung);
        }
        catch (SendSubscriptionNotConfirmed e) {
            debug.error(String.format("Es sind Probleme beim Senden der Statusinformationsdaten aufgetreten. %s", e));
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 28.11.12
